'use client'

import React, { useState, useEffect } from 'react';
import { Button, notification, Tag } from 'antd';
import { DeleteOutlined, LeftOutlined } from '@ant-design/icons';
import dayjs from 'dayjs';
import Image from 'next/image';
import { useRouter } from '../../../i18n/routing';
import { Report, Room, RoomsType } from '../../../types/rooms';
import { i18nText } from '../../../i18nKeys';
import { LinkButton } from '../../view/LinkButton';
import {
    addClient,
    addSupervisor,
    becomeRoomClient,
    becomeRoomSupervisor,
    deleteRoomClient,
    deleteRoomSupervisor,
    getReport
} from '../../../actions/rooms';
import { AUTH_TOKEN_KEY, AUTH_USER } from '../../../constants/common';
import { useLocalStorage } from '../../../hooks/useLocalStorage';
import { UserListModal } from '../../Modals/UsersListModal';
import { SessionState } from '../../../types/sessions';
import { EditRoomForm } from './EditRoomForm';

type RoomDetailsContentProps = {
    locale: string;
    activeType: RoomsType;
    room?: Room;
    startRoom: () => void;
    refresh: () => void;
};

export const RoomDetailsContent = ({ room, startRoom, locale, activeType, refresh }: RoomDetailsContentProps) => {
    const [jwt] = useLocalStorage(AUTH_TOKEN_KEY, '');
    const [userData] = useLocalStorage(AUTH_USER, '');
    const { id: userId = 0 } = userData ? JSON.parse(userData) : {};
    const router = useRouter();
    const [showModal, setShowModal] = useState<boolean>(false);
    const [forSupervisor, setForSupervisor] = useState<boolean>(false);
    const startDate = room?.scheduledStartAtUtc ? dayjs(room?.scheduledStartAtUtc).locale(locale) : null;
    const endDate = room?.scheduledEndAtUtc ? dayjs(room?.scheduledEndAtUtc).locale(locale) : null;
    const today = startDate ? dayjs().format('YYYY-MM-DD') === startDate.format('YYYY-MM-DD') : false;
    const isCreator = room?.coach && room.coach.id === +userId || false;
    const isSupervisor = room?.supervisor && room.supervisor.id === +userId || false;
    const isClient = room?.clients && room.clients.length > 0 && room.clients.map(({ id }) => id).includes(+userId) || false;
    const isTimeBeforeStart = room?.scheduledStartAtUtc ? dayjs() < dayjs(room.scheduledStartAtUtc) : false;
    const [isEdit, setIsEdit] = useState<boolean>(false);
    const [report, setReport] = useState<Report[] | undefined>();

    useEffect(() => {
        if (room?.id && room?.supervisor && activeType === RoomsType.RECENT) {
            getReport(locale, jwt, room.id)
                .then((data) => {
                    setReport(data);
                })
        }
    }, [room])

    const goBack = () => router.push(`/account/rooms/${activeType}`);

    const checkUserApply = (): boolean => (!room?.supervisor || !isSupervisor) && (!room?.clients || room?.clients && room?.clients.length === 0 || !isClient);

    const deleteClient = (clientUserId: number) => {
        if (room?.id) {
            deleteRoomClient(locale, jwt, { sessionId: room.id, clientUserId })
                .then(() => {
                    refresh();
                })
                .catch((err) => {
                    notification.error({
                        message: 'Error',
                        description: err?.response?.data?.errMessage
                    });
                });
        }
    };

    const deleteSupervisor = (supervisorUserId?: number) => {
        if (room?.id && supervisorUserId) {
            deleteRoomSupervisor(locale, jwt, { sessionId: room.id, supervisorUserId })
                .then(() => {
                    refresh();
                })
                .catch((err) => {
                    notification.error({
                        message: 'Error',
                        description: err?.response?.data?.errMessage
                    });
                })
        }
    };

    const becomeClient = () => {
        if (room?.id && userId) {
            becomeRoomClient(locale, jwt, { sessionId: room.id, clientUserId: +userId })
                .then(() => {
                    refresh();
                })
                .catch((err) => {
                    notification.error({
                        message: 'Error',
                        description: err?.response?.data?.errMessage
                    });
                });
        }
    };

    const becomeSupervisor = () => {
        if (room?.id && userId) {
            becomeRoomSupervisor(locale, jwt, { sessionId: room.id, supervisorUserId: +userId })
                .then(() => {
                    refresh();
                })
                .catch((err) => {
                    notification.error({
                        message: 'Error',
                        description: err?.response?.data?.errMessage
                    });
                });
        }
    };

    const onInviteSupervisor = () => {
        setForSupervisor(true)
        setShowModal(true);
    };

    const onAddUser = (id: number) => {
        if (room?.id) {
            setShowModal(false);

            if (forSupervisor) {
                addSupervisor(locale, jwt, { sessionId: room.id, supervisorUserId: id })
                    .then(() => {
                        refresh();
                    })
                    .catch((err) => {
                        notification.error({
                            message: 'Error',
                            description: err?.response?.data?.errMessage
                        });
                    });
            } else {
                addClient(locale, jwt, { sessionId: room.id, clientUserId: id })
                    .then(() => {
                        refresh();
                    })
                    .catch((err) => {
                        notification.error({
                            message: 'Error',
                            description: err?.response?.data?.errMessage
                        });
                    });
            }
        }
    };

    const afterEditing = () => {
        setIsEdit(false);
        refresh();
    }

    return !isEdit ? (
        <div className="card-detail">
            <div>
                <Button
                    className="card-detail__back"
                    type="link"
                    icon={<LeftOutlined/>}
                    onClick={goBack}
                >
                    {i18nText('back', locale)}
                </Button>
            </div>
            <div className="card-detail__name">{room?.title || ''}</div>
            <div
                className={`card-detail__date${today ? ' chosen' : ''}${activeType === RoomsType.RECENT ? ' history' : ''}`}>
                {today
                    ? `${i18nText('today', locale)} ${startDate?.format('HH:mm')} - ${endDate?.format('HH:mm')}`
                    : `${startDate?.format('D MMMM')} ${startDate?.format('HH:mm')} - ${endDate?.format('HH:mm')}`}
            </div>
            {room?.themesTags && room.themesTags.length > 0 && (
                <div className="card-detail__skills">
                    <div className="skills__list">
                        {room.themesTags.map((skill) => <Tag key={skill?.id}
                                                               className="skills__list__item">{skill?.name}</Tag>)}
                    </div>
                </div>
            )}
            {room?.description && <div className="card-profile__desc">{room.description}</div>}
            {activeType === RoomsType.UPCOMING && (isCreator || isSupervisor || isClient) && (
                <div className="card-detail__actions">
                    {(isCreator || isClient || isSupervisor) && (
                        <Button
                            className="card-detail__apply"
                            onClick={startRoom}
                        >
                            {isCreator ? i18nText('session.start', locale) : i18nText('session.join', locale)}
                        </Button>
                    )}
                    {isCreator && isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && (
                        <Button
                            className="card-detail__filled"
                            onClick={() => setIsEdit(true)}
                        >
                            {i18nText('room.editRoom', locale)}
                        </Button>
                    )}
                </div>
            )}
            <div className="card-detail__profile">
                <div className="card-detail__profile_title">
                    <div>{i18nText('room.roomCreator', locale)}</div>
                </div>
                <div className="card-detail__profile_list">
                    <div className="card-detail__profile_item">
                        <div className="card-detail__portrait card-detail__portrait_small">
                            <Image src={room?.coach?.faceImageUrl || '/images/user-avatar.png'} width={86} height={86} alt=""/>
                        </div>
                        <div className="card-detail__inner">
                            <div className="card-detail__name">{`${room?.coach?.name} ${room?.coach?.surname || ''}`}</div>
                        </div>
                    </div>
                </div>
            </div>
            {room?.isNeedSupervisor && (
                <div className="card-detail__profile">
                    <div className="card-detail__profile_title">
                        <div>{i18nText('room.supervisor', locale)}</div>
                    </div>
                    {room?.supervisor && (
                        <div className="card-detail__profile_list">
                            <div className="card-detail__profile_item">
                                <div className="card-detail__portrait card-detail__portrait_small">
                                    <Image src={room?.supervisor?.faceImageUrl || '/images/user-avatar.png'} width={86}
                                           height={86}
                                           alt=""/>
                                </div>
                                <div className="card-detail__inner">
                                    <div
                                        className="card-detail__name">{`${room?.supervisor?.name} ${room?.supervisor?.surname || ''}`}</div>
                                </div>
                                {isCreator && activeType === RoomsType.UPCOMING && isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && (
                                    <LinkButton
                                        type="link"
                                        style={{alignSelf: 'flex-start'}}
                                        danger
                                        icon={<DeleteOutlined/>}
                                        onClick={() => deleteSupervisor(room?.supervisor?.id)}
                                    />
                                )}
                            </div>
                        </div>
                    )}
                    {room?.supervisor && activeType === RoomsType.RECENT && (
                        <>
                            {room?.supervisorComment && (
                                <div className="card-detail__supervisor-comment">{room.supervisorComment}</div>
                            )}
                            {report && report.length > 0 && (
                                <div className="card-detail__report-list">
                                    {report.map(({ key, score }) => (
                                        <div key={key}>
                                            <div>{i18nText(`room.rating_${key?.toLowerCase()}`, locale)}</div>
                                            <div className="card-detail__report-list_divider" />
                                            <div>{score || 0}</div>
                                        </div>
                                    ))}
                                </div>
                            )}
                        </>
                    )}
                    {isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && !room?.supervisor && isCreator && activeType === RoomsType.UPCOMING && (
                        <Button
                            className="card-detail__filled"
                            onClick={onInviteSupervisor}
                        >
                            {i18nText('room.inviteSupervisor', locale)}
                        </Button>
                    )}
                    {isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && !room?.supervisor && !isCreator && activeType === RoomsType.UPCOMING && checkUserApply() && (
                        <Button
                            className="card-detail__apply"
                            onClick={becomeSupervisor}
                        >
                            {i18nText('room.joinSupervisor', locale)}
                        </Button>
                    )}
                    {!room?.supervisor && !isCreator && !checkUserApply() && (
                        <div className="card-profile__desc">{i18nText('noData', locale)}</div>
                    )}
                </div>
            )}
            <div className="card-detail__profile">
                <div className="card-detail__profile_title">
                    <div>{i18nText('room.participants', locale)}</div>
                    <div>{`${room?.clients?.length || 0}/${room?.maxClients}`}</div>
                </div>
                {room?.clients && room?.clients?.length > 0 && (
                    <div className="card-detail__profile_list">
                        {room.clients.map(({id, faceImageUrl, name, surname}) => (
                            <div key={id} className="card-detail__profile_item">
                                <div className="card-detail__portrait card-detail__portrait_small">
                                    <Image src={faceImageUrl || '/images/user-avatar.png'} width={86}
                                           height={86}
                                           alt=""/>
                                </div>
                                <div className="card-detail__inner">
                                    <div
                                        className="card-detail__name">{`${name} ${surname || ''}`}</div>
                                </div>
                                {isCreator && room?.state === SessionState.COACH_APPROVED && activeType === RoomsType.UPCOMING && isTimeBeforeStart && (
                                    <LinkButton
                                        type="link"
                                        style={{alignSelf: 'flex-start'}}
                                        danger
                                        icon={<DeleteOutlined/>}
                                        onClick={() => deleteClient(id)}
                                    />
                                )}
                            </div>
                        ))}
                    </div>
                )}
                {isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && isCreator && activeType === RoomsType.UPCOMING && (!room?.clients || (room?.clients && room?.clients?.length < room.maxClients)) && (
                    <Button
                        className="card-detail__filled"
                        onClick={() => setShowModal(true)}
                    >
                        {i18nText('room.inviteParticipant', locale)}
                    </Button>
                )}
                {isTimeBeforeStart && room?.state === SessionState.COACH_APPROVED && !isCreator && activeType === RoomsType.UPCOMING && (!room?.clients || (room?.clients && room?.clients?.length < room.maxClients)) && checkUserApply() && (
                    <Button
                        className="card-detail__apply"
                        onClick={becomeClient}
                    >
                        {i18nText('room.joinParticipant', locale)}
                    </Button>
                )}
            </div>
            {room && (
                <UserListModal
                    locale={locale}
                    jwt={jwt}
                    isOpen={showModal}
                    handleCancel={() => setShowModal(false)}
                    submit={onAddUser}
                    afterCloseModal={() => setForSupervisor(false)}
                    room={room}
                />
            )}
        </div>
    ) : (
        <div className="card-detail">
            <div>
                <Button
                    className="card-detail__back"
                    type="link"
                    icon={<LeftOutlined/>}
                    onClick={() => setIsEdit(false)}
                >
                    {i18nText('back', locale)}
                </Button>
            </div>
            <EditRoomForm
                roomId={room?.id || 0}
                locale={locale}
                jwt={jwt}
                mode="edit"
                afterSubmit={afterEditing}
            />
        </div>
    );
};
